@@ -13,6 +13,8 @@ from typing import Union
 from cleo import option
 from tomlkit import inline_table
 
+from poetry.core.pyproject import PyProjectException
+from poetry.core.pyproject.toml import PyProjectTOML
 from poetry.utils._compat import OrderedDict
 from poetry.utils._compat import Path
 from poetry.utils._compat import urlparse
@@ -22,7 +24,6 @@ from .env_command import EnvCommand
 
 
 class InitCommand(Command):
-
     name = "init"
     description = (
         "Creates a basic <comment>pyproject.toml</> file in the current directory."
@@ -32,6 +33,7 @@ class InitCommand(Command):
         option("name", None, "Name of the package.", flag=False),
         option("description", None, "Description of the package.", flag=False),
         option("author", None, "Author name of the package.", flag=False),
+        option("python", None, "Compatible Python versions.", flag=False),
         option(
             "dependency",
             None,
@@ -61,14 +63,25 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         self._pool = None
 
     def handle(self):
+        from poetry.core.vcs.git import GitConfig
         from poetry.layouts import layout
         from poetry.utils._compat import Path
         from poetry.utils.env import SystemEnv
-        from poetry.vcs.git import GitConfig
 
-        if (Path.cwd() / "pyproject.toml").exists():
-            self.line("<error>A pyproject.toml file already exists.</error>")
-            return 1
+        pyproject = PyProjectTOML(Path.cwd() / "pyproject.toml")
+
+        if pyproject.file.exists():
+            if pyproject.is_poetry_project():
+                self.line(
+                    "<error>A pyproject.toml file with a poetry section already exists.</error>"
+                )
+                return 1
+
+            if pyproject.data.get("build-system"):
+                self.line(
+                    "<error>A pyproject.toml file with a defined build-system already exists.</error>"
+                )
+                return 1
 
         vcs_config = GitConfig()
 
@@ -126,21 +139,27 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         question.set_validator(self._validate_license)
         license = self.ask(question)
 
-        current_env = SystemEnv(Path(sys.executable))
-        default_python = "^{}".format(
-            ".".join(str(v) for v in current_env.version_info[:2])
-        )
-        question = self.create_question(
-            "Compatible Python versions [<comment>{}</comment>]: ".format(
-                default_python
-            ),
-            default=default_python,
-        )
-        python = self.ask(question)
+        python = self.option("python")
+        if not python:
+            current_env = SystemEnv(Path(sys.executable))
+            default_python = "^{}".format(
+                ".".join(str(v) for v in current_env.version_info[:2])
+            )
+            question = self.create_question(
+                "Compatible Python versions [<comment>{}</comment>]: ".format(
+                    default_python
+                ),
+                default=default_python,
+            )
+            python = self.ask(question)
 
         self.line("")
 
         requirements = {}
+        if self.option("dependency"):
+            requirements = self._format_requirements(
+                self._determine_requirements(self.option("dependency"))
+            )
 
         question = "Would you like to define your main dependencies interactively?"
         help_message = (
@@ -157,12 +176,16 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         if self.confirm(question, True):
             self.line(help_message)
             help_displayed = True
-            requirements = self._format_requirements(
-                self._determine_requirements(self.option("dependency"))
+            requirements.update(
+                self._format_requirements(self._determine_requirements([]))
             )
             self.line("")
 
         dev_requirements = {}
+        if self.option("dev-dependency"):
+            dev_requirements = self._format_requirements(
+                self._determine_requirements(self.option("dev-dependency"))
+            )
 
         question = (
             "Would you like to define your development dependencies interactively?"
@@ -171,8 +194,8 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
             if not help_displayed:
                 self.line(help_message)
 
-            dev_requirements = self._format_requirements(
-                self._determine_requirements(self.option("dev-dependency"))
+            dev_requirements.update(
+                self._format_requirements(self._determine_requirements([]))
             )
             self.line("")
 
@@ -187,7 +210,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
             dev_dependencies=dev_requirements,
         )
 
-        content = layout_.generate_poetry_content()
+        content = layout_.generate_poetry_content(original=pyproject)
         if self.io.is_interactive():
             self.line("<info>Generated file</info>")
             self.line("")
@@ -203,7 +226,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
             f.write(content)
 
     def _determine_requirements(
-        self, requires, allow_prereleases=False
+        self, requires, allow_prereleases=False, source=None
     ):  # type: (List[str], bool) -> List[Dict[str, str]]
         if not requires:
             requires = []
@@ -259,6 +282,10 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
                         attempts=3,
                     )
 
+                    # package selected by user, set constraint name to package name
+                    if package is not False:
+                        constraint["name"] = package
+
                 # no constraint yet, determine the best version automatically
                 if package is not False and "version" not in constraint:
                     question = self.create_question(
@@ -299,7 +326,9 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
             elif "version" not in requirement:
                 # determine the best version automatically
                 name, version = self._find_best_version_for_package(
-                    requirement["name"], allow_prereleases=allow_prereleases
+                    requirement["name"],
+                    allow_prereleases=allow_prereleases,
+                    source=source,
                 )
                 requirement["version"] = version
                 requirement["name"] = name
@@ -314,6 +343,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
                     requirement["name"],
                     requirement["version"],
                     allow_prereleases=allow_prereleases,
+                    source=source,
                 )
 
                 requirement["name"] = name
@@ -323,13 +353,13 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         return result
 
     def _find_best_version_for_package(
-        self, name, required_version=None, allow_prereleases=False
+        self, name, required_version=None, allow_prereleases=False, source=None
     ):  # type: (...) -> Tuple[str, str]
         from poetry.version.version_selector import VersionSelector
 
         selector = VersionSelector(self._get_pool())
         package = selector.find_best_candidate(
-            name, required_version, allow_prereleases=allow_prereleases
+            name, required_version, allow_prereleases=allow_prereleases, source=source
         )
 
         if not package:
@@ -349,13 +379,13 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
 
         try:
             cwd = self.poetry.file.parent
-        except RuntimeError:
+        except (PyProjectException, RuntimeError):
             cwd = Path.cwd()
 
         for requirement in requirements:
             requirement = requirement.strip()
             extras = []
-            extras_m = re.search(r"\[([\w\d,-_]+)\]$", requirement)
+            extras_m = re.search(r"\[([\w\d,-_ ]+)\]$", requirement)
             if extras_m:
                 extras = [e.strip() for e in extras_m.group(1).split(",")]
                 requirement, _ = requirement.split("[")
@@ -364,8 +394,8 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
             if url_parsed.scheme and url_parsed.netloc:
                 # Url
                 if url_parsed.scheme in ["git+https", "git+ssh"]:
-                    from poetry.vcs.git import Git
-                    from poetry.vcs.git import ParsedUrl
+                    from poetry.core.vcs.git import Git
+                    from poetry.core.vcs.git import ParsedUrl
 
                     parsed = ParsedUrl.parse(requirement)
                     url = Git.normalize_url(requirement)
@@ -378,7 +408,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
                         pair["extras"] = extras
 
                     package = Provider.get_package_from_vcs(
-                        "git", url.url, reference=pair.get("rev")
+                        "git", url.url, rev=pair.get("rev")
                     )
                     pair["name"] = package.name
                     result.append(pair)
@@ -478,7 +508,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         return requires
 
     def _validate_author(self, author, default):
-        from poetry.packages.package import AUTHOR_REGEX
+        from poetry.core.packages.package import AUTHOR_REGEX
 
         author = author or default
 
@@ -495,7 +525,7 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         return author
 
     def _validate_license(self, license):
-        from poetry.spdx import license_by_id
+        from poetry.core.spdx import license_by_id
 
         if license:
             license_by_id(license)
