@@ -7,6 +7,7 @@ from clikit.api.io import IO
 from poetry.poetry import Poetry
 from poetry.utils._compat import Path
 from poetry.utils._compat import decode
+from poetry.utils._compat import urlparse
 
 
 class Exporter(object):
@@ -139,6 +140,9 @@ class Exporter(object):
                 url = (
                     repository.authenticated_url if with_credentials else repository.url
                 )
+                parsed_url = urlparse.urlsplit(url)
+                if parsed_url.scheme == "http":
+                    indexes_header += "--trusted-host {}\n".format(parsed_url.netloc)
                 indexes_header += "--extra-index-url {}\n".format(url)
 
             content = indexes_header + "\n" + content
